// LCD.H                            Schweitzer                       11.10.2011
// ----------------------------------------------------------------------------

//	Ansteuerung des LC-Display.

// In der Headerdatei "zeit.h" muss eine Funktion zeit_us (unsigned int)
// vorhanden sein. Diese wird fr kurze Zeitverzgerungen bei der Display-
// steuerung bentigt. ("zeit.h" muss vor "lcd.h" aufgerufen werden)

// ============================================================================

// ----------------------------------------------------------------------------
// Definitionen
// ----------------------------------------------------------------------------

#define	lcd_port			PORTC			// Verwendeter Port
#define  lcd_dir			DDRC			// Datenrichtung (Eing. oder Ausg.)
#define  lcd_pin			PINC			//	Verwendete Eingangspins

// ----------------------------------------------------------------------------
// Bentigte Headerdateien
// ----------------------------------------------------------------------------

#include "ZEIT.H"	// Zeitfunktionen

// ----------------------------------------------------------------------------
// Funktionsprototypen
// ----------------------------------------------------------------------------

// Pins  deklarieren
void lcd_rs (unsigned char);		// Register select (0: Steuerung, 1: Daten)
void lcd_rw (unsigned char);		// Read/Write      (0: schreiben, 1: lesen)
void lcd_e  (unsigned char);		// Enable, muss getaktet werden: 0 > 1 > 0
void lcd_led (unsigned char);	 	// Hintergrundbeleuchtung, 1: einschalten
	
// Hilfs-Funktionen deklarieren
void takten (void);					// LCD takten
void ausgabe (char);					// Datenbertragung an LCD
void warten (void);					// Prfen, ob LCD beschftigt (busy)

// Steuerbytes zum LCD
void lcd_steuerbyte (char);			

// Daten zum LCD (Anzeige)
void lcd_datenbyte (char);				

// LCD initialisieren
void lcd_initialisieren (void);					
void lcd_clear (void);					


// Text auf LC-Display
void lcd_text (unsigned char, unsigned char, char[]);	
void lcd_zahl (unsigned char, unsigned char, unsigned short, unsigned char);

//-----------------------------------------------------------------------------
// Register Select (0: Steuerbyte wird bertragen, 1: Datenbyte wird bertr.)
//-----------------------------------------------------------------------------

void lcd_rs (unsigned char wert)
{

	lcd_dir = 0xff;							// PE0...PE7 als Ausgnge definieren

	if (wert == 0) lcd_port = lcd_pin & 0xfe;	// Pin 0 zurcksetzen, Steuerb.
	if (wert == 1) lcd_port = lcd_pin | 0x01;	// Pin 0 setzen, Datenbyte
	
}

//-----------------------------------------------------------------------------
// Read/Write      (0: in das Display schreiben, 1: vom Display lesen)
//-----------------------------------------------------------------------------

void lcd_rw (unsigned char wert)
{

	if (wert == 0) lcd_port = lcd_pin & 0xfd;	// Pin 1 zurcksetzen, write
	if (wert == 1) lcd_port = lcd_pin | 0x02;	// Pin 1 setzen, read
	
}

//-----------------------------------------------------------------------------
// Enable, muss getaktet werden
//-----------------------------------------------------------------------------

void lcd_e (unsigned char wert)
{

	if (wert == 0) lcd_port = lcd_pin & 0xfb;	// Pin 2 zurcksetzen
	if (wert == 1) lcd_port = lcd_pin | 0x04;	// Pin 2 setzen
	
}

//-----------------------------------------------------------------------------
// Hintergrundbeleuchtung, 1: einschalten
//-----------------------------------------------------------------------------

void lcd_led (unsigned char wert)
{

	if (wert == 0) lcd_port = lcd_pin & 0xf7;	// Pin 3 zurcksetzen
	if (wert == 1) lcd_port = lcd_pin | 0x08;	// Pin 3 setzen
	
}

//-----------------------------------------------------------------------------
// LC-Display takten
//-----------------------------------------------------------------------------

void takten (void)
{

	lcd_e (1);											// Enable 1
	zeit_us (1);										// Pause
	lcd_e (0);											// Enable 0
	zeit_us (1);										// Pause
	
}

//-----------------------------------------------------------------------------
// Wert der Variablen "zeichen" zum Display bertragen
//-----------------------------------------------------------------------------

void ausgabe (char zeichen)
{

	lcd_rw(0);								// LCD in Schreibmodus setzen

	// Die unteren vier Datenbits lschen und mit den Steuerbits verknpfen.
	// Ergebnis an den lcd-Port ausgeben
	lcd_port = ((zeichen & 0xf0) | (lcd_pin & 0x0f));
	takten ();
	
	// Die unteren vier Datenbits nach links schieben und mit den Steuerbits 
	// verknpfen. Ergebnis an den lcd-Port ausgeben
	lcd_port = (( (zeichen << 4) & 0xf0) | (lcd_pin & 0x0f));
	takten ();
	
	warten ();
	
}

//-----------------------------------------------------------------------------
// Warten bis LC-Display nicht mehr beschftigt
//-----------------------------------------------------------------------------

void warten()
{

	char temp;								// temporres Hilfsbyte

	lcd_rw (1);								// Lesemodus (read)
	lcd_rs (0);								// Steuerbefehl folgt
	lcd_dir = 0x0f;						// obere 4 Bits als Eingang definieren

	do
	{
		lcd_e (1);							// 1. Lesevorgang
		zeit_us (1);
		temp  = lcd_pin & 0x80;			// Busy-Flag lesen
		lcd_e (0);							// Freigabe takten fr
		zeit_us (1);
		takten ();								// 2. Lesevorgang. Ohne Funktion
	}
	while (temp == 0x80);				// Schleife bis LCD frei

	lcd_dir = 0xff;						// Port als Ausgang definieren
	
}

//-----------------------------------------------------------------------------
// Ausgabe von Steuerbefehlen an LC-Display
//-----------------------------------------------------------------------------

void lcd_steuerbyte(char zeichen)
{

	lcd_rs (0);									// LCD auf Steuerbefehl vorbereiten
	ausgabe (zeichen);						// Steuercode an LCD senden
	
}

//-----------------------------------------------------------------------------
// Ausgabe von Daten an LC-Display
//-----------------------------------------------------------------------------

void lcd_datenbyte(char zeichen)
{

	lcd_rs (1);									// LCD auf Datenbyte vorbereiten
	ausgabe (zeichen);						// Datenbyte an LCD senden
	
}


//-----------------------------------------------------------------------------
// LC-Display initialisieren
//-----------------------------------------------------------------------------

void lcd_initialisieren(void)
{

	zeit(40);									// Warten bis Display bereit

	lcd_dir  = 0xff;							// Initialisierungssequenz nach Datenblatt
	lcd_port = 0x3f;
	zeit(5);
	warten();									// Busy-Flag abfragen

	lcd_rw (0);									// LCD in Schreibmodus versetzen
	lcd_rs (0);									// LCD auf Steuerbefehl vorbereiten
	warten();									// Busy-Flag abfragen

	lcd_steuerbyte (0x20);					// Function Set, 4 Bit-Modus
	lcd_steuerbyte (0x28);					// Function Set, Zweizeiliges Display
	lcd_steuerbyte (0x80);					// Function Set, Zweizeiliges Display
	lcd_steuerbyte (0x01);					// Display lschen
	lcd_steuerbyte (0x02);					// Cursor Home
	lcd_steuerbyte (0x0c);					// Display an, Cursor aus
	
	lcd_led (0);								// Hintergrundbeleuchtung ausschalten

}

//-----------------------------------------------------------------------------
// LC-Display lschen
//-----------------------------------------------------------------------------

void lcd_clear(void)
{
	lcd_steuerbyte(0x01);					// Display lschen (clear)
}

//-----------------------------------------------------------------------------
// Text auf LC-Display ausgeben
//-----------------------------------------------------------------------------

void lcd_text(unsigned char zeile, unsigned char stelle, char *text)
{
	int adresse, i = 0;
	adresse = 128 + 64 * (zeile - 1) + stelle; 

	lcd_steuerbyte (adresse);

	while (text[i] !=0)                 // Endlosschleife
	{
		switch (text[i])
		{
			case '':
			lcd_datenbyte (0xe1);
			break;
			case '':
			lcd_datenbyte (0xef);
			break;
			case '':
			lcd_datenbyte (0xf5);
			break;
			case '':
			lcd_datenbyte (0xe1);
			break;
			case '':
			lcd_datenbyte (0xef);
			break;
			case '':
			lcd_datenbyte (0xf5);
			break;
			case '':
			lcd_datenbyte (0xe2);
			break;
		
			default:
			lcd_datenbyte(text[i]);

		}
		i++;
	}

}

//-----------------------------------------------------------------------------
// Zahl auf LC-Display ausgeben
//-----------------------------------------------------------------------------

void lcd_zahl(unsigned char zeile, unsigned char stelle, unsigned short zahl, unsigned char laenge)
{

	unsigned char zehntausender, tausender, hunderter, zehner, einer, position;

	position = (128 + (zeile - 1) * 64) + stelle;		// Cursor-Position berechnen

	zehntausender = (zahl - (zahl % 10000)) / 10000;	// Berechnet die Anzahl der Zehntausender in einer Zahl
	zahl = zahl - zehntausender * 10000;					// Zehntausender aus 'zahl' herausrechnen
	zehntausender = zehntausender + 0x30;					// Wandelt Ziffer in ASCII-Code

	tausender = (zahl - (zahl % 1000)) / 1000;			// Berechnet die Anzahl der Tausender in einer Zahl
	zahl = zahl - tausender * 1000;							// Tausender aus 'zahl' herausrechnen
	tausender = tausender + 0x30;								// Wandelt Ziffer in ASCII-Code

	hunderter = (zahl - (zahl % 100)) / 100;				// Berechnet die Anzahl der Hunderter in einer Zahl
	zahl = zahl - hunderter * 100;							// Hunderter aus 'zahl' herausrechnen
	hunderter = hunderter + 0x30;								// Wandelt Ziffer in ASCII-Code

	zehner = (zahl - (zahl % 10)) / 10;						// Berechnet die Anzahl der Zehner in einer Zahl
	zahl= zahl - zehner * 10;									// Zehner aus 'zahl' herausrechnen
	zehner = zehner + 0x30;										// Wandelt Ziffer in ASCII-Code

	einer = zahl + 0x30;											// In zahl stehen nur noch Einer

	lcd_steuerbyte(position);									// Cursor auf Position bewegen

	if(laenge > 4) lcd_datenbyte(zehntausender);			// Zehntausender ausgeben
	if(laenge > 3) lcd_datenbyte(tausender);				// Tausender ausgeben
	if(laenge > 2) lcd_datenbyte(hunderter);				// Hunderter ausgeben
	if(laenge > 1) lcd_datenbyte(zehner);					// Zehner ausgeben
	lcd_datenbyte(einer);										// Einer ausgeben

}

// ============================================================================
